"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for details.
Object.defineProperty(exports, "__esModule", { value: true });
/// <reference path="exponentHelper.d.ts" />
const path = require("path");
const Q = require("q");
const XDL = require("./xdlInterface");
const package_1 = require("../../common/node/package");
const reactNativeProjectHelper_1 = require("../../common/reactNativeProjectHelper");
const fileSystem_1 = require("../../common/node/fileSystem");
const OutputChannelLogger_1 = require("../log/OutputChannelLogger");
const stripJSONComments = require("strip-json-comments");
const APP_JSON = "app.json";
const EXP_JSON = "exp.json";
const EXPONENT_INDEX = "exponentIndex.js";
const DEFAULT_EXPONENT_INDEX = "index.js";
const DEFAULT_IOS_INDEX = "index.ios.js";
const DEFAULT_ANDROID_INDEX = "index.android.js";
const DBL_SLASHES = /\\/g;
class ExponentHelper {
    constructor(workspaceRootPath, projectRootPath) {
        this.logger = OutputChannelLogger_1.OutputChannelLogger.getMainChannel();
        this.workspaceRootPath = workspaceRootPath;
        this.projectRootPath = projectRootPath;
        this.hasInitialized = false;
        // Constructor is slim by design. This is to add as less computation as possible
        // to the initialization of the extension. If a public method is added, make sure
        // to call this.lazilyInitialize() at the begining of the code to be sure all variables
        // are correctly initialized.
    }
    configureExponentEnvironment() {
        this.lazilyInitialize();
        this.logger.info("Making sure your project uses the correct dependencies for exponent. This may take a while...");
        return this.isExpoApp(true)
            .then(isExpo => {
            this.logger.logStream(".\n");
            return this.patchAppJson(isExpo);
        });
    }
    /**
     * Returns the current user. If there is none, asks user for username and password and logins to exponent servers.
     */
    loginToExponent(promptForInformation, showMessage) {
        this.lazilyInitialize();
        return XDL.currentUser()
            .then((user) => {
            if (!user) {
                let username = "";
                return showMessage("You need to login to exponent. Please provide username and password to login. If you don't have an account we will create one for you.")
                    .then(() => promptForInformation("Exponent username", false)).then((name) => {
                    username = name;
                    return promptForInformation("Exponent password", true);
                })
                    .then((password) => XDL.login(username, password));
            }
            return user;
        })
            .catch(error => {
            return Q.reject(error);
        });
    }
    getExpPackagerOptions() {
        this.lazilyInitialize();
        return this.getFromExpConfig("packagerOpts")
            .then(opts => opts || {});
    }
    /**
     * Path to a given file inside the .vscode directory
     */
    dotvscodePath(filename) {
        return path.join(this.workspaceRootPath, ".vscode", filename);
    }
    createExpoEntry(name) {
        this.lazilyInitialize();
        return this.detectEntry()
            .then((entryPoint) => {
            const content = this.generateFileContent(name, entryPoint);
            return this.fs.writeFile(this.dotvscodePath(EXPONENT_INDEX), content);
        });
    }
    detectEntry() {
        this.lazilyInitialize();
        return Q.all([
            this.fs.exists(this.pathToFileInWorkspace(DEFAULT_EXPONENT_INDEX)),
            this.fs.exists(this.pathToFileInWorkspace(DEFAULT_IOS_INDEX)),
            this.fs.exists(this.pathToFileInWorkspace(DEFAULT_ANDROID_INDEX)),
        ])
            .spread((expo, ios) => {
            return expo ? this.pathToFileInWorkspace(DEFAULT_EXPONENT_INDEX) :
                ios ? this.pathToFileInWorkspace(DEFAULT_IOS_INDEX) :
                    this.pathToFileInWorkspace(DEFAULT_ANDROID_INDEX);
        });
    }
    generateFileContent(name, entryPoint) {
        return `// This file is automatically generated by VS Code
// Please do not modify it manually. All changes will be lost.
var React = require('${this.pathToFileInWorkspace("/node_modules/react")}');
var { Component } = React;
var ReactNative = require('${this.pathToFileInWorkspace("/node_modules/react-native")}');
var { AppRegistry } = ReactNative;
var entryPoint = require('${entryPoint}');
AppRegistry.registerRunnable('main', function(appParameters) {
    AppRegistry.runApplication('${name}', appParameters);
});`;
    }
    patchAppJson(isExpo = true) {
        return this.readAppJson()
            .catch(() => {
            // if app.json doesn't exist but it's ok, we will create it
            return {};
        })
            .then((config) => {
            let expoConfig = (config.expo || {});
            if (!expoConfig.name || !expoConfig.slug) {
                return this.getPackageName()
                    .then((name) => {
                    expoConfig.slug = expoConfig.slug || config.name || name.replace(" ", "-");
                    expoConfig.name = expoConfig.name || config.name || name;
                    config.expo = expoConfig;
                    return config;
                });
            }
            return config;
        })
            .then((config) => {
            if (!config.expo.sdkVersion) {
                return this.exponentSdk(true)
                    .then(sdkVersion => {
                    config.expo.sdkVersion = sdkVersion;
                    return config;
                });
            }
            return config;
        })
            .then((config) => {
            if (!isExpo) {
                config.expo.entryPoint = this.dotvscodePath(EXPONENT_INDEX);
            }
            return config;
        })
            .then((config) => {
            return config ? this.writeAppJson(config) : config;
        })
            .then((config) => {
            return isExpo ? Q.resolve(void 0) : this.createExpoEntry(config.expo.name);
        });
    }
    /**
     * Exponent sdk version that maps to the current react-native version
     * If react native version is not supported it returns null.
     */
    exponentSdk(showProgress = false) {
        if (showProgress) {
            this.logger.logStream("...");
        }
        return reactNativeProjectHelper_1.ReactNativeProjectHelper.getReactNativeVersion(this.projectRootPath)
            .then(version => {
            if (showProgress)
                this.logger.logStream(".");
            return XDL.mapVersion(version)
                .then(sdkVersion => {
                if (!sdkVersion) {
                    return XDL.supportedVersions()
                        .then((versions) => {
                        return Q.reject(new Error(`React Native version not supported by exponent. Major versions supported: ${versions.join(", ")}`));
                    });
                }
                return sdkVersion;
            });
        });
    }
    /**
     * Name specified on user's package.json
     */
    getPackageName() {
        return new package_1.Package(this.projectRootPath, { fileSystem: this.fs }).name();
    }
    getExpConfig() {
        return this.readExpJson()
            .catch(err => {
            if (err.code === "ENOENT") {
                return this.readAppJson()
                    .then((config) => {
                    return config.expo || {};
                });
            }
            return err;
        });
    }
    getFromExpConfig(key) {
        return this.getExpConfig()
            .then((config) => config[key]);
    }
    /**
     * Returns the specified setting from exp.json if it exists
     */
    readExpJson() {
        const expJsonPath = this.pathToFileInWorkspace(EXP_JSON);
        return this.fs.readFile(expJsonPath)
            .then(content => {
            return JSON.parse(stripJSONComments(content));
        });
    }
    readAppJson() {
        const appJsonPath = this.pathToFileInWorkspace(APP_JSON);
        return this.fs.readFile(appJsonPath)
            .then(content => {
            return JSON.parse(stripJSONComments(content));
        });
    }
    writeAppJson(config) {
        const appJsonPath = this.pathToFileInWorkspace(APP_JSON);
        return this.fs.writeFile(appJsonPath, JSON.stringify(config, null, 2))
            .then(() => config);
    }
    /**
     * Path to a given file from the workspace root
     */
    pathToFileInWorkspace(filename) {
        return path.join(this.projectRootPath, filename).replace(DBL_SLASHES, "/");
    }
    isExpoApp(showProgress = false) {
        this.logger.logStream("Checking if this is Expo app.");
        if (showProgress) {
            this.logger.logStream("...");
        }
        const packageJsonPath = this.pathToFileInWorkspace("package.json");
        return this.fs.readFile(packageJsonPath)
            .then(content => {
            const packageJson = JSON.parse(content);
            const isExp = packageJson.dependencies && !!packageJson.dependencies.expo || false;
            if (showProgress)
                this.logger.logStream(".");
            return isExp;
        }).catch(() => {
            if (showProgress) {
                this.logger.logStream(".");
            }
            // Not in a react-native project
            return false;
        });
    }
    /**
     * Works as a constructor but only initiliazes when it's actually needed.
     */
    lazilyInitialize() {
        if (!this.hasInitialized) {
            this.hasInitialized = true;
            this.fs = new fileSystem_1.FileSystem();
            XDL.configReactNativeVersionWargnings();
            XDL.attachLoggerStream(this.projectRootPath, {
                stream: {
                    write: (chunk) => {
                        if (chunk.level <= 30) {
                            this.logger.logStream(chunk.msg);
                        }
                        else if (chunk.level === 40) {
                            this.logger.warning(chunk.msg);
                        }
                        else {
                            this.logger.error(chunk.msg);
                        }
                    },
                },
                type: "raw",
            });
        }
    }
}
exports.ExponentHelper = ExponentHelper;

//# sourceMappingURL=exponentHelper.js.map
